==> generator-0.0.50-sources.jar: cn/bestwu/generator/dsl/Generator.kt <==
package cn.bestwu.generator.dsl

import cn.bestwu.generator.DatabaseDriver
import cn.bestwu.generator.GeneratorException
import cn.bestwu.generator.GeneratorExtension
import cn.bestwu.generator.database.domain.Column
import cn.bestwu.generator.database.domain.Indexed
import cn.bestwu.generator.database.domain.Table
import cn.bestwu.generator.dom.java.JavaType
import cn.bestwu.generator.dom.java.JavaTypeResolver
import cn.bestwu.generator.dom.java.PrimitiveTypeWrapper
import org.atteo.evo.inflector.English
import java.io.File
import java.io.PrintWriter

/**
 * 模板基类
 */
abstract class Generator {
    companion object {
        const val DEFAULT_NAME = "generated.txt"

        fun toBoolean(obj: Any?): Boolean {
            when (obj) {
                is Boolean -> return obj
                is String -> {
                    if (obj.equals("true", true)) {
                        return true
                    }
                    when (obj.length) {
                        1 -> {
                            val ch0 = obj[0]
                            if (ch0 == 'y' || ch0 == 'Y' ||
                                    ch0 == 't' || ch0 == 'T' || ch0 == '1') {
                                return true
                            }
                            if (ch0 == 'n' || ch0 == 'N' ||
                                    ch0 == 'f' || ch0 == 'F' || ch0 == '0') {
                                return false
                            }
                        }
                        2 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            if ((ch0 == 'o' || ch0 == 'O') && (ch1 == 'n' || ch1 == 'N')) {
                                return true
                            }
                            if ((ch0 == 'n' || ch0 == 'N') && (ch1 == 'o' || ch1 == 'O')) {
                                return false
                            }
                        }
                        3 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            if ((ch0 == 'y' || ch0 == 'Y') &&
                                    (ch1 == 'e' || ch1 == 'E') &&
                                    (ch2 == 's' || ch2 == 'S')) {
                                return true
                            }
                            if ((ch0 == 'o' || ch0 == 'O') &&
                                    (ch1 == 'f' || ch1 == 'F') &&
                                    (ch2 == 'f' || ch2 == 'F')) {
                                return false
                            }
                        }
                        4 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            val ch3 = obj[3]
                            if ((ch0 == 't' || ch0 == 'T') &&
                                    (ch1 == 'r' || ch1 == 'R') &&
                                    (ch2 == 'u' || ch2 == 'U') &&
                                    (ch3 == 'e' || ch3 == 'E')) {
                                return true
                            }
                        }
                        5 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            val ch3 = obj[3]
                            val ch4 = obj[4]
                            if ((ch0 == 'f' || ch0 == 'F') &&
                                    (ch1 == 'a' || ch1 == 'A') &&
                                    (ch2 == 'l' || ch2 == 'L') &&
                                    (ch3 == 's' || ch3 == 'S') &&
                                    (ch4 == 'e' || ch4 == 'E')) {
                                return false
                            }
                        }
                        else -> {
                        }
                    }
                }
                is Number -> return obj.toInt() > 0
            }

            return false
        }

    }

    protected lateinit var table: Table
    protected open val extension: GeneratorExtension
        get() = Generators.extension
    protected val isOracleDatasource
        get() = extension.datasource.databaseDriver == DatabaseDriver.ORACLE

    protected val basePackageName: String
        get() = extension.basePackageName

    open var cover: Boolean = false
    protected open val test: Boolean = false

    protected open val dir: String
        get() = if (test) extension.dir.replace("src/main/", "src/test/") else extension.dir

    protected val basePath: File
        get() = extension.basePath ?: File("")

    protected open val path: File
        get() = File(basePath, dir)

    /**
     * 文件名称
     */
    protected open val name: String = DEFAULT_NAME

    protected open val destFile: File
        get() {
            var outputPath = path
            if (moduleName.isNotBlank()) {
                outputPath = File(path, moduleName)
            }
            return File(outputPath, if (this is JavaGenerator) "${name.replace(".", "/")}.java" else name)
        }

    protected open val moduleName: String
        get() = extension.moduleName

    protected open val moduleComment: String
        get() = extension.moduleComment

    protected val settings: Map<String, Any?>
        get() = extension.settings

    protected fun property(key: String): Any? = settings[key]

    protected fun <T> property(key: String, default: T): T {
        val any = settings[key]
        return if (any == null) {
            return default
        } else {
            @Suppress("UNCHECKED_CAST")
            any as T
        }
    }

    protected val className
        get() = extension.className(tableName)

    protected val entityName
        get() = className.decapitalize()

    protected val Column.javaType
        get() = JavaTypeResolver.calculateJavaType(this)!!

    protected val Column.jdbcType: String
        get() = JavaTypeResolver.calculateJdbcTypeName(this)!!

    protected open val Column.randomValue: Any
        get() = when (javaType) {
            JavaType("java.sql.Timestamp") -> System.currentTimeMillis()
            JavaType.dateInstance, JavaType("java.sql.Date") -> System.currentTimeMillis()
            JavaType("java.sql.Time") -> System.currentTimeMillis()
            PrimitiveTypeWrapper.booleanInstance -> true
            PrimitiveTypeWrapper.doubleInstance -> 1.0
            PrimitiveTypeWrapper.longInstance -> 1L
            PrimitiveTypeWrapper.integerInstance -> 1
            JavaType.stringInstance -> "test"
            else -> 1
        }

    protected open val Column.randomValueToSet: String
        get() = when (javaType) {
            JavaType("java.math.BigDecimal") -> "new java.math.BigDecimal(\"1.0\")"
            JavaType("java.sql.Timestamp") -> "new java.sql.Timestamp(System.currentTimeMillis())"
            JavaType.dateInstance -> "new java.util.Date(System.currentTimeMillis())"
            JavaType("java.sql.Date") -> "new java.sql.Date(System.currentTimeMillis())"
            JavaType("java.sql.Time") -> "new java.sql.Time(System.currentTimeMillis())"
            PrimitiveTypeWrapper.booleanInstance -> "true"
            PrimitiveTypeWrapper.doubleInstance -> "1.0"
            PrimitiveTypeWrapper.longInstance -> "1L"
            PrimitiveTypeWrapper.integerInstance -> "1"
            PrimitiveTypeWrapper.shortInstance -> "new Short(\"1\")"
            PrimitiveTypeWrapper.byteInstance -> "new Byte(\"1\")"
            JavaType("byte[]") -> "new byte[0]"
            JavaType.stringInstance -> "\"test\""
            else -> "1"
        }

    protected val Column.testId: Any
        get() = when (javaType) {
            JavaType.stringInstance -> "\"1\""
            PrimitiveTypeWrapper.longInstance -> "1L"
            PrimitiveTypeWrapper.integerInstance -> 1
            else -> 1
        }

    protected val Column.javaName
        get() = extension.javaName(this.columnName)

    protected val Column.initializationString
        get() = when {
            javaType.shortName == "Boolean" -> toBoolean(defaultVal).toString()
            javaType.shortName == "Long" -> "${defaultVal}L"
            javaType.shortName == "Double" -> "${defaultVal}D"
            javaType.shortName == "Float" -> "${defaultVal}F"
            javaType.shortName == "BigDecimal" -> "new BigDecimal($defaultVal)"
            javaType.shortName == "String" -> "\"$defaultVal\""
            defaultVal == "CURRENT_TIMESTAMP" -> "new Date()"
            else -> defaultVal
        }

    /**
     * 表名
     */
    protected val tableName: String
        get() = table.tableName

    protected val catalog: String?
        get() = table.catalog

    protected val schema: String?
        get() = table.schema

    /**
     * 表类型
     */
    protected val tableType: String
        get() = table.tableType

    /**
     * 注释说明
     */
    protected val comment: String
        get() {
            val comment = if (table.comment.endsWith("表")) table.comment.substringBeforeLast("表") else table.comment
            return if (comment.isBlank()) {
                extension.comment
            } else {
                comment
            }
        }
    /**
     * 主键
     */
    protected val primaryKeys: List<Column>
        get() {
            val primaryKeys = table.primaryKeys
            return if (primaryKeys.isEmpty()) {
                val primaryKey = columns.find { it.columnName.equals(extension.primaryKeyName, true) }
                        ?: columns.find { it.comment.contains("主键") }
                if (primaryKey != null) {
                    listOf(primaryKey)
                } else {
                    emptyList()
                }
            } else {
                primaryKeys
            }
        }
    /**
     * 主键
     */
    protected val primaryKey: Column
        get() {
            if (primaryKeys.size == 1) {
                return primaryKeys[0]
            } else {
                throw GeneratorException("$tableName:没有单主键，$primaryKeyNames")
            }
        }
    /**
     * 字段
     */
    protected val columns: List<Column>
        get() = table.columns

    protected val indexes: List<Indexed>
        get() = table.indexes

    protected val primaryKeyNames: List<String>
        get() = primaryKeys.map { it.columnName }

    protected val Column.isPrimary
        get() = primaryKeyNames.contains(columnName)

    protected val pathName: String
        get() = English.plural(entityName)

    fun call(table: Table): Any? {
        this.table = table
        return if (supports())
            doCall()
        else
            null
    }

    protected open fun supports(): Boolean {
        return true
    }

    protected open fun output(printWriter: PrintWriter) {

    }

    open fun setUp() {

    }

    open fun tearDown() {

    }

    protected open fun doCall() {
        if (destFile.exists() && ((!extension.replaceAll && !cover) || destFile.readLines().any { it.contains("[[Don't cover]]") })) {
            return
        }
        destFile.parentFile.mkdirs()

        val msg = if (destFile.exists()) "覆盖" else "生成"
        destFile.printWriter().use {
            output(it)
        }
        println("$msg：$destFile")
    }
}

==> generator-0.0.53-sources.jar: cn/bestwu/generator/dsl/Generator.kt <==
package cn.bestwu.generator.dsl

import cn.bestwu.generator.DatabaseDriver
import cn.bestwu.generator.GeneratorException
import cn.bestwu.generator.GeneratorExtension
import cn.bestwu.generator.database.domain.Column
import cn.bestwu.generator.database.domain.Indexed
import cn.bestwu.generator.database.domain.Table
import java.io.File
import java.io.PrintWriter

/**
 * 模板基类
 */
abstract class Generator {
    companion object {
        const val DEFAULT_NAME = "generated.txt"

        fun toBoolean(obj: Any?): Boolean {
            when (obj) {
                is Boolean -> return obj
                is String -> {
                    if (obj.equals("true", true)) {
                        return true
                    }
                    when (obj.length) {
                        1 -> {
                            val ch0 = obj[0]
                            if (ch0 == 'y' || ch0 == 'Y' ||
                                    ch0 == 't' || ch0 == 'T' || ch0 == '1') {
                                return true
                            }
                            if (ch0 == 'n' || ch0 == 'N' ||
                                    ch0 == 'f' || ch0 == 'F' || ch0 == '0') {
                                return false
                            }
                        }
                        2 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            if ((ch0 == 'o' || ch0 == 'O') && (ch1 == 'n' || ch1 == 'N')) {
                                return true
                            }
                            if ((ch0 == 'n' || ch0 == 'N') && (ch1 == 'o' || ch1 == 'O')) {
                                return false
                            }
                        }
                        3 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            if ((ch0 == 'y' || ch0 == 'Y') &&
                                    (ch1 == 'e' || ch1 == 'E') &&
                                    (ch2 == 's' || ch2 == 'S')) {
                                return true
                            }
                            if ((ch0 == 'o' || ch0 == 'O') &&
                                    (ch1 == 'f' || ch1 == 'F') &&
                                    (ch2 == 'f' || ch2 == 'F')) {
                                return false
                            }
                        }
                        4 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            val ch3 = obj[3]
                            if ((ch0 == 't' || ch0 == 'T') &&
                                    (ch1 == 'r' || ch1 == 'R') &&
                                    (ch2 == 'u' || ch2 == 'U') &&
                                    (ch3 == 'e' || ch3 == 'E')) {
                                return true
                            }
                        }
                        5 -> {
                            val ch0 = obj[0]
                            val ch1 = obj[1]
                            val ch2 = obj[2]
                            val ch3 = obj[3]
                            val ch4 = obj[4]
                            if ((ch0 == 'f' || ch0 == 'F') &&
                                    (ch1 == 'a' || ch1 == 'A') &&
                                    (ch2 == 'l' || ch2 == 'L') &&
                                    (ch3 == 's' || ch3 == 'S') &&
                                    (ch4 == 'e' || ch4 == 'E')) {
                                return false
                            }
                        }
                        else -> {
                        }
                    }
                }
                is Number -> return obj.toInt() > 0
            }

            return false
        }

    }

    protected lateinit var table: Table
    protected open val extension: GeneratorExtension
        get() = Generators.extension
    protected val isOracleDatasource
        get() = extension.datasource.databaseDriver == DatabaseDriver.ORACLE

    protected val basePackageName: String
        get() = extension.basePackageName

    open var cover: Boolean = false
    protected open val test: Boolean = false

    protected open val dir: String
        get() = if (test) extension.dir.replace("src/main/", "src/test/") else extension.dir

    protected val basePath: File
        get() = extension.basePath ?: File("")

    protected open val path: File
        get() = File(basePath, dir)

    /**
     * 文件名称
     */
    protected open val name: String = DEFAULT_NAME

    protected open val destFile: File
        get() {
            var outputPath = path
            if (module.isNotBlank()) {
                outputPath = File(path, module)
            }
            return File(outputPath, if (this is JavaGenerator) "${name.replace(".", "/")}.java" else name)
        }

    open var module: String = ""
        get() = if (extension.module.isNotBlank()) extension.module else field

    open var moduleName: String = ""
        get() = if (extension.moduleName.isNotBlank()) extension.moduleName else field

    protected open val projectName: String
        get() = extension.projectName

    protected val settings: Map<String, Any?>
        get() = extension.settings

    protected fun property(key: String): Any? = settings[key]

    protected fun <T> property(key: String, default: T): T {
        val any = settings[key]
        return if (any == null) {
            return default
        } else {
            @Suppress("UNCHECKED_CAST")
            any as T
        }
    }

    protected open val className
        get() = table.className

    protected val entityName
        get() = table.entityName

    /**
     * 表名
     */
    protected val tableName: String
        get() = table.tableName

    protected val catalog: String?
        get() = table.catalog

    protected val schema: String?
        get() = table.schema

    /**
     * 表类型
     */
    protected val tableType: String
        get() = table.tableType

    /**
     * 注释说明
     */
    protected val remarks: String
        get() {
            val comment = if (table.remarks.endsWith("表")) table.remarks.substringBeforeLast("表") else table.remarks
            return if (comment.isBlank()) {
                extension.remarks
            } else {
                comment
            }
        }
    /**
     * 主键
     */
    protected val primaryKeys: List<Column>
        get() {
            val primaryKeys = table.primaryKeys
            return if (primaryKeys.isEmpty()) {
                val primaryKey = columns.find { it.columnName.equals(extension.primaryKeyName, true) }
                        ?: columns.find { it.remarks.contains("主键") }
                if (primaryKey != null) {
                    listOf(primaryKey)
                } else {
                    emptyList()
                }
            } else {
                primaryKeys
            }
        }
    /**
     * 主键
     */
    protected val primaryKey: Column
        get() {
            if (primaryKeys.size == 1) {
                return primaryKeys[0]
            } else {
                throw GeneratorException("$tableName:没有单主键，$primaryKeyNames")
            }
        }
    /**
     * 字段
     */
    protected val columns: List<Column>
        get() = table.columns

    protected val indexes: List<Indexed>
        get() = table.indexes

    protected val primaryKeyNames: List<String>
        get() = table.primaryKeyNames

    protected val pathName: String
        get() = table.pathName

    fun call(table: Table): Any? {
        this.table = table
        return if (supports())
            doCall()
        else
            null
    }

    protected open fun supports(): Boolean {
        return true
    }

    protected open fun output(printWriter: PrintWriter) {

    }

    open fun setUp() {

    }

    open fun tearDown() {

    }

    protected open fun doCall() {
        if (destFile.exists() && ((!extension.replaceAll && !cover) || destFile.readLines().any { it.contains("[[Don't cover]]") })) {
            return
        }
        destFile.parentFile.mkdirs()

        val msg = if (destFile.exists()) "覆盖" else "生成"
        destFile.printWriter().use {
            output(it)
        }
        println("$msg：$destFile")
    }
}
